/*
 * Copyright (C) 2000 John David Ratliff, Jason Andrew Fisk, and Hongwei Guan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package model;

import datastructures.DoublyLinkedList;
import java.util.StringTokenizer;

//////////////////////////////////////////////////////////////////////
/**
 * This class represents one line in our editor.  A line is made up of
 * a list of words.  Each of these words (even if they have the same
 * value, are represented as unique objects.
 *
 *@see tools.Word
 *@see datastructures.DoublyLinkedList
 */
//////////////////////////////////////////////////////////////////////
public class Line
    {
    //The words that make up this line.
    private DoublyLinkedList words;
    private int lineNumber;

    //////////////////////////////////////////////////////////////////////
    /**
     * In this constructor each word of the given string is parsed out into
     * separate words.
     *
     *@param string The string representation of this line.
     */
    public Line(String line)
        {
	StringTokenizer parser = new StringTokenizer(line);
	words = new DoublyLinkedList();
	
	//march through the string and create new words for each word in
	//the line. 
	String currentString;
	int wordIndex = 0;
	Word currentWord;
	while(parser.hasMoreTokens())
            {
	    currentString = parser.nextToken();
	    
	    //create a new word given the current string, this line, and
	    //the current index.
	    currentWord = new Word(currentString, this);
	    words.insert(wordIndex,currentWord);
	    wordIndex++;
            }
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Insert the given word into this line at the index of the word.  
     * NOTE: We must then shift the indices of the words to the right 
     * of the given word.
     */
    public void insertWord(Word word, int index)
        throws IndexOutOfBoundsException
        {
        if (index < 0 || index > size())
            {
            throw new IndexOutOfBoundsException();
            }
        else
            {
            words.insert(index, word);
            }
        }

    //////////////////////////////////////////////////////////////////////    
    /**
     * Delete the given word from this line. 
     * NOTE: We must then shift the indices of the words to the right 
     * of the given word.
     */
    public void deleteWord(Word word)
        {
	if(contains(word))
            {
	    int index = indexOf(word);
	    words.delete(index);
            }
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Returns true if this line contains this word.  Otherwise, false.
     */
    public boolean contains(Word word)
        {
	return (words.indexOf(word) != -1);
        }

    //////////////////////////////////////////////////////////////////////    
    /**
     * Return the first index of the given word in this line.  It returns
     * -1 if it is not in this line.
     */
    public int indexOf(Word word)
        {
	return words.indexOf(word);
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Return the word at the given index.  Throws an
     * IndexOutOfBoundsException if index in out of range.
     */
    public Word getWordAt(int index)
        {
	//see if it is out of range.
	if(index < 0 || index >= size())
            {
            throw new IndexOutOfBoundsException();
            }

	//Otherwise return what is at that index.
	return (Word)words.get(index);
        }
    
    //////////////////////////////////////////////////////////////////////
    /**
     * Return the length of this line.
     */
    public int size()
        {
	return words.size();
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Return the words of this line concatenated together with a space
     * between them.
     */
    public String toString()
        {
	String totalValue = "";
	for(int i = 0 ; i < size(); i++)
            {
	    totalValue += words.get(i).toString();
	    totalValue +=" ";
            }
	totalValue.trim();
	return totalValue;
        }
    }
			 
	    
	



