/*
 * Copyright (C) 2000 John David Ratliff, Jason Andrew Fisk, and Hongwei Guan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package view;

import java.awt.BorderLayout;
import java.awt.Color;
import javax.swing.JFrame;
import javax.swing.JTextPane;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.ImageIcon;

import controller.MVC_Controller;
import controller.Controller;

import driver.AutomationObject;

import model.MVC_Model;

import view.MVC_View;

/* The implementation of the View. It implements the MVC_View interface and uses the updateView() method to
	process input provided by the Controller.

	The display uses various highlighting colors to show various aspects of the Document:
	RED: Errors and Status Label Commands (the statusLabel)
	BLACK, non-bold: Standard text, no special meaning
	BLACK, bold: Highlighted text from $DISPLAY_LINES
	BLUE: The current line
	GREEN: The words found by $FIND_WORD
*/

public class VisualEditor implements MVC_View {
	private MVC_Model model;

	private JFrame frame;
	private JTextPane textPane;
	private JTextField textField;
	private JLabel commandLabel;
	private JLabel statusLabel;
	private ImageIcon errorIcon;
	private JPanel panel;

	// Frame constants
	private final static int FRAME_WIDTH = 640;
	private final static int FRAME_HEIGHT = 480;
	private final static String APPLICATION_TITLE = "Simple Visual LineEditor 1.5";

	// Component constants
	private final static int TEXT_WIDTH = 30;
	private final static String COMMAND_LABEL = "Command: ";
	private final static String ERROR_ICON = "images/error.gif";

	// initilizes the VisualEditor, creates the MVC_Model (LineManager), and creates the
	// display (initDisplay() -- the JFrame and sub-components)
	public VisualEditor(AutomationObject automation, MVC_Model model) {
		this.model = model;

		this.initDisplay(automation);
	}

	// creates the JFrame and sub-components, and the Controller
	private void initDisplay(AutomationObject automation) {
		// create the controller
		MVC_Controller controller = new Controller(this.model);

		// create the frame
		this.frame = new JFrame();

		// create the JTextPane. Make it an HTML Document
		this.textPane = new JTextPane(new HTMLDocument());
		this.textPane.setEditorKit(new HTMLEditorKit());
		this.textPane.setEditable(false);

		// create a textField. Ask the controller to listen to the textField
		this.textField = new JTextField(VisualEditor.TEXT_WIDTH);
		controller.handleTextFieldEvents(this.textField);

		// create the error Icon (displayed when an ERROR occurs on the staus label)
		this.errorIcon = new ImageIcon(VisualEditor.ERROR_ICON);

		// create the status label. errors/statuses are displayed in <font color=Red>RED</font>
		this.statusLabel = new JLabel(" ");
		this.statusLabel.setForeground(Color.red);

		// create the command label (the label marked "Command: ")
		this.commandLabel = new JLabel(VisualEditor.COMMAND_LABEL);
		this.commandLabel.setForeground(Color.black);

		// create a sub-panel to make BorderLayout more useful
		// add the textField, statusLabel, and commandLabel to it.
		this.panel = new JPanel(new BorderLayout());
		panel.add(this.textField, BorderLayout.CENTER);
		panel.add(this.commandLabel, BorderLayout.WEST);
		panel.add(this.statusLabel, BorderLayout.SOUTH);

		// set the frame properties, and add the textPane and the sub-panel to it
		frame.setTitle(VisualEditor.APPLICATION_TITLE);
		frame.setSize(VisualEditor.FRAME_WIDTH, VisualEditor.FRAME_HEIGHT);
		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(this.textPane, BorderLayout.CENTER);
		frame.getContentPane().add(this.panel, BorderLayout.SOUTH);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);

		// give focus to the textField and give AutomationObject the textField
		this.textField.requestFocus();
		automation.setTextField(this.textField);
	}

	// update the view with the new data
	// the controller passes the data to the view, which passes it to the model
	public void updateView(String buffer) {
		this.clearStatus();
		this.textPane.setText(buffer);
	}

	// display a message on the statusLabel. add the error icon if isError == true
	public void updateView(String message, boolean isError) {
		this.statusLabel.setText(message);

		if (isError) {
			this.statusLabel.setIcon(this.errorIcon);
		} else {
			this.statusLabel.setIcon(null);
		}
	}

	// clear the staus label (but keep is visible)
	private void clearStatus() {
		this.statusLabel.setText(" ");
		this.statusLabel.setIcon(null);
	}
}
